use std::fs::read_to_string;

fn read_as_vectors(input: &str) -> (Vec<i32>, Vec<i32>) {
    let input = read_to_string(input).unwrap();
    let (left, right): (Vec<_>, Vec<_>) = input
        .lines()
        .map(|x| {
            let mut l = x.split_whitespace().map(|n| n.parse::<i32>().unwrap());
            (l.next().unwrap(), l.next().unwrap())
        })
        .unzip();
    (left, right)
}

pub fn stage1(input: &str) -> i32 {
    let (mut left, mut right) = read_as_vectors(input);
    left.sort();
    right.sort();
    let res: i32 = left
        .iter()
        .zip(right.iter())
        .map(|(l, r)| (l - r).abs())
        .sum();
    res
}

pub fn stage2(input: &str) -> i32 {
    let (left, right) = read_as_vectors(input);
    left.iter()
        .map(|l| right.iter().filter(|&f| *f == *l).count() as i32 * l)
        .sum()
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_stage1() {
        assert_eq!(stage1("input/01_input.txt"), 1590491);
    }

    #[test]
    fn test_stage2() {
        assert_eq!(stage2("input/01_input.txt"), 22588371);
    }
}
