use std::fs::read_to_string;

use regex::Regex;

pub fn stage1(input: &str) -> usize {
    let r = Regex::new(r"mul\((\d*),(\d*)\)").unwrap();
    read_to_string(input)
        .unwrap()
        .lines()
        .map(|line| {
            r.captures_iter(line)
                .map(|c| {
                    let x: usize = c[1].parse().unwrap();
                    let y: usize = c[2].parse().unwrap();
                    x * y
                })
                .collect::<Vec<usize>>()
        })
        .flatten()
        .sum()
}

pub fn stage2(input: &str) -> usize {
    let r = Regex::new(r"(mul\((\d*),(\d*)\))|(don't\(\))|(do\(\))").unwrap();
    let mut state = true;
    read_to_string(input)
        .unwrap()
        .lines()
        .map(|line| {
            r.captures_iter(line)
                .map(|c| {
                    if c[0].starts_with("mul") && state {
                        let x: usize = c[2].parse().unwrap();
                        let y: usize = c[3].parse().unwrap();
                        return x * y;
                    }
                    if c[0].starts_with("don't()") {
                        state = false;
                    }
                    if c[0].starts_with("do()") {
                        state = true;
                    }

                    0
                })
                .collect::<Vec<usize>>()
        })
        .flatten()
        .sum()
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_stage1() {
        assert_eq!(stage1("input/03_input.txt"), 165225049);
    }

    #[test]
    fn test_stage2() {
        assert_eq!(stage2("input/03_input.txt"), 108830766);
    }
}
